"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.commandHandler = exports.builder = exports.desc = exports.command = void 0;
const ern_core_1 = require("ern-core");
const ern_cauldron_api_1 = require("ern-cauldron-api");
const ern_composite_gen_1 = require("ern-composite-gen");
const ern_container_gen_1 = require("ern-container-gen");
const lib_1 = require("../../lib");
const path_1 = __importDefault(require("path"));
exports.command = 'upload';
exports.desc = 'Upload a bundle to a store';
const builder = (argv) => {
    return argv
        .option('baseComposite', {
        describe: 'Base Composite',
        type: 'string',
    })
        .coerce('baseComposite', (d) => ern_core_1.PackagePath.fromString(d))
        .option('descriptor', {
        alias: 'd',
        describe: 'Full native application descriptor',
        type: 'string',
    })
        .coerce('descriptor', (d) => ern_core_1.AppVersionDescriptor.fromString(d))
        .option('extraJsDependencies', {
        alias: 'e',
        describe: 'Additional JS dependency(ies)',
        type: 'array',
    })
        .coerce('extraJsDependencies', (d) => d.map(ern_core_1.PackagePath.fromString))
        .option('fromGitBranches', {
        describe: 'Favor MiniApp(s) branches',
        type: 'boolean',
    })
        .option('fromPackager', {
        describe: 'Upload bundle served by current local packager',
        type: 'boolean',
    })
        .option('jsApiImpls', {
        describe: 'One or more JS API implementation(s)',
        type: 'array',
    })
        .coerce('jsApiImpls', (d) => d.map(ern_core_1.PackagePath.fromString))
        .option('miniapps', {
        alias: 'm',
        describe: 'One or more MiniApp(s)',
        type: 'array',
    })
        .option('platform', {
        choices: ['android', 'ios', undefined],
        describe: 'If set will only upload the bundle for this platform (otherwise will upload for both platforms)',
        type: 'string',
    })
        .option('prod', {
        describe: 'Set to upload a production bundle rather than a development one',
        type: 'boolean',
    })
        .option('resetCache', {
        describe: 'Indicates whether to reset the React Native cache prior to bundling',
        type: 'boolean',
    })
        .coerce('miniapps', (d) => d.map(ern_core_1.PackagePath.fromString))
        .epilog((0, lib_1.epilog)(exports));
};
exports.builder = builder;
const commandHandler = async ({ baseComposite, descriptor, extraJsDependencies, fromGitBranches, fromPackager, jsApiImpls, miniapps, platform, prod, resetCache, } = {}) => {
    await (0, lib_1.logErrorAndExitIfNotSatisfied)({
        bundleStoreAccessKeyIsSet: {
            extraErrorMessage: `You can use:
'ern bundlestore use <accessKey>' to use an existing store OR
'ern bundlestore create <storeName>' to create a new store`,
        },
        bundleStoreUrlSetInCauldron: {
            extraErrorMessage: `You should add bundleStore config in your Cauldron`,
        },
    });
    const platforms = platform
        ? [platform]
        : ['android', 'ios'];
    const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
    const bundleStoreUrl = (await cauldron.getBundleStoreConfig()).url;
    const engine = new ern_core_1.BundleStoreEngine(bundleStoreUrl);
    if (fromPackager) {
        for (const curPlatform of platforms) {
            const bundleId = await ern_core_1.kax
                .task(`Uploading ${curPlatform} bundle`)
                .run(engine.uploadFromPackager({ platform: curPlatform, dev: !prod }));
            ern_core_1.log.info(`Successfully uploaded ${curPlatform} bundle [id: ${bundleId}]`);
        }
    }
    else {
        // Full native application descriptor was not provided.
        // Ask the user to select a completeNapDescriptor from a list
        // containing all the native applications versions in the cauldron
        // Not needed if miniapps are directly provided
        if (!descriptor && !miniapps) {
            descriptor = await (0, lib_1.askUserToChooseANapDescriptorFromCauldron)();
        }
        let pathToYarnLock;
        let resolutions;
        let metroExtraNodeModules;
        if (descriptor) {
            await (0, lib_1.logErrorAndExitIfNotSatisfied)({
                napDescriptorExistInCauldron: {
                    descriptor,
                    extraErrorMessage: 'You cannot create a composite for a non-existing native application version.',
                },
            });
            miniapps = await cauldron.getContainerMiniApps(descriptor, {
                favorGitBranches: !!fromGitBranches,
            });
            jsApiImpls = await cauldron.getContainerJsApiImpls(descriptor);
            const containerGenConfig = await cauldron.getContainerGeneratorConfig(descriptor);
            if (!containerGenConfig || !containerGenConfig.bypassYarnLock) {
                pathToYarnLock = await cauldron.getPathToYarnLock(descriptor, 'container');
            }
            else {
                ern_core_1.log.debug('Bypassing yarn.lock usage as bypassYarnLock flag is set in config');
            }
            const compositeGenConfig = await cauldron.getCompositeGeneratorConfig(descriptor);
            baseComposite =
                baseComposite ||
                    ((compositeGenConfig === null || compositeGenConfig === void 0 ? void 0 : compositeGenConfig.baseComposite) &&
                        ern_core_1.PackagePath.fromString(compositeGenConfig.baseComposite));
            resolutions = compositeGenConfig === null || compositeGenConfig === void 0 ? void 0 : compositeGenConfig.resolutions;
            metroExtraNodeModules = compositeGenConfig === null || compositeGenConfig === void 0 ? void 0 : compositeGenConfig.metroExtraNodeModules;
        }
        const compositeDir = (0, ern_core_1.createTmpDir)();
        await ern_core_1.kax.task('Generating Composite').run((0, ern_composite_gen_1.generateComposite)({
            baseComposite,
            extraJsDependencies,
            jsApiImplDependencies: jsApiImpls,
            metroExtraNodeModules,
            miniApps: miniapps,
            outDir: compositeDir,
            pathToYarnLock,
            resolutions,
        }));
        for (const curPlatform of platforms) {
            const outDir = (0, ern_core_1.createTmpDir)();
            const bundlePath = path_1.default.join(outDir, 'index.bundle');
            const sourceMapPath = path_1.default.join(outDir, 'index.map');
            await ern_core_1.kax.task(`Bundling MiniApps for ${curPlatform}`).run((0, ern_container_gen_1.bundleMiniAppsFromComposite)({
                bundleOutput: bundlePath,
                compositeDir,
                dev: !prod,
                outDir,
                platform: curPlatform,
                resetCache,
                sourceMapOutput: path_1.default.join(outDir, 'index.map'),
            }));
            const bundleId = await ern_core_1.kax
                .task(`Uploading ${curPlatform} bundle`)
                .run(engine.upload({ bundlePath, platform: curPlatform, sourceMapPath }));
            ern_core_1.log.info(`Successfully uploaded ${curPlatform} bundle [id: ${bundleId}]`);
        }
    }
};
exports.commandHandler = commandHandler;
exports.handler = (0, lib_1.tryCatchWrap)(exports.commandHandler);
//# sourceMappingURL=upload.js.map