"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.commandHandler = exports.builder = exports.desc = exports.command = void 0;
const ern_core_1 = require("ern-core");
const ern_cauldron_api_1 = require("ern-cauldron-api");
const ern_orchestrator_1 = require("ern-orchestrator");
const lib_1 = require("../../../lib");
exports.command = 'miniapps <miniapps..>';
exports.desc = 'Remove one or more MiniApp(s) from the cauldron';
const builder = (argv) => {
    return argv
        .option('containerVersion', {
        alias: 'v',
        describe: 'Version to use for generated container. If none provided, version will be patch bumped by default',
        type: 'string',
    })
        .option('descriptor', {
        alias: 'd',
        describe: 'A complete native application descriptor',
        type: 'string',
    })
        .coerce('descriptor', (d) => ern_core_1.AppVersionDescriptor.fromString(d))
        .coerce('miniapps', (d) => d.map(ern_core_1.PackagePath.fromString))
        .option('resetCache', {
        describe: 'Indicates whether to reset the React Native cache prior to bundling',
        type: 'boolean',
    })
        .epilog((0, lib_1.epilog)(exports));
};
exports.builder = builder;
// This command does not actually removes or offers to remove dependencies that were
// only used by this MiniApp
// It could be done as a future improvement to this command
const commandHandler = async ({ containerVersion, descriptor, miniapps, resetCache, }) => {
    descriptor =
        descriptor ||
            (await (0, lib_1.askUserToChooseANapDescriptorFromCauldron)({
                onlyNonReleasedVersions: true,
            }));
    await (0, lib_1.logErrorAndExitIfNotSatisfied)({
        isNewerContainerVersion: containerVersion
            ? {
                containerVersion,
                descriptor,
                extraErrorMessage: 'To avoid conflicts with previous versions, you can only use container version newer than the current one',
            }
            : undefined,
        miniAppIsInNativeApplicationVersionContainer: {
            descriptor,
            extraErrorMessage: 'This command cannot remove MiniApp(s) that do not exist in Cauldron.',
            miniApp: miniapps,
        },
        napDescriptorExistInCauldron: {
            descriptor,
            extraErrorMessage: 'This command cannot work on a non existing native application version',
        },
    });
    const cauldronCommitMessage = [
        `${miniapps.length === 1
            ? `Remove ${miniapps[0]} MiniApp from ${descriptor}`
            : `Remove multiple MiniApps from ${descriptor}`}`,
    ];
    if (!(await (0, lib_1.emptyContainerIfSingleMiniAppOrJsApiImpl)(descriptor))) {
        const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
        await (0, ern_orchestrator_1.syncCauldronContainer)(async () => {
            for (const miniapp of miniapps) {
                await cauldron.removeMiniAppFromContainer(descriptor, miniapp);
                cauldronCommitMessage.push(`- Remove ${miniapp} MiniApp`);
            }
        }, descriptor, cauldronCommitMessage, { containerVersion, resetCache });
    }
    ern_core_1.log.info(`MiniApp(s) successfully removed from ${descriptor}`);
};
exports.commandHandler = commandHandler;
exports.handler = (0, lib_1.tryCatchWrap)(exports.commandHandler);
//# sourceMappingURL=miniapps.js.map