"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.commandHandler = exports.builder = exports.desc = exports.command = void 0;
const ern_core_1 = require("ern-core");
const ern_cauldron_api_1 = require("ern-cauldron-api");
const ern_orchestrator_1 = require("ern-orchestrator");
const lib_1 = require("../../lib");
const untildify_1 = __importDefault(require("untildify"));
exports.command = 'regen-container';
exports.desc = 'Triggers the regeneration of a Container from the Cauldron';
const builder = (argv) => {
    return argv
        .option('containerVersion', {
        alias: 'v',
        describe: 'Version to use for generated container. If none provided, version will be patched bumped by default.',
        type: 'string',
    })
        .option('descriptor', {
        alias: 'd',
        describe: 'A complete native application descriptor',
        type: 'string',
    })
        .coerce('descriptor', (d) => ern_core_1.AppVersionDescriptor.fromString(d))
        .option('fullRegen', {
        describe: 'Perform complete regeneration',
        type: 'boolean',
    })
        .option('resetCache', {
        describe: 'Indicates whether to reset the React Native cache prior to bundling',
        type: 'boolean',
    })
        .option('sourceMapOutput', {
        describe: 'Path to source map file to generate for this container bundle',
        type: 'string',
    })
        .coerce('sourceMapOutput', (p) => (0, untildify_1.default)(p))
        .epilog((0, lib_1.epilog)(exports));
};
exports.builder = builder;
const commandHandler = async ({ containerVersion, descriptor, fullRegen, resetCache, sourceMapOutput, }) => {
    descriptor =
        descriptor ||
            (await (0, lib_1.askUserToChooseANapDescriptorFromCauldron)({
                onlyNonReleasedVersions: true,
            }));
    await (0, lib_1.logErrorAndExitIfNotSatisfied)({
        isNewerContainerVersion: containerVersion
            ? {
                containerVersion,
                descriptor,
                extraErrorMessage: 'To avoid conflicts with previous versions, you can only use container version newer than the current one',
            }
            : undefined,
        napDescriptorExistInCauldron: {
            descriptor,
            extraErrorMessage: 'This command cannot work on a non existing native application version',
        },
    });
    const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
    // Figure out the list of all git MiniApps that have been updated (new HEAD sha)
    const updatedGitMiniApps = await cauldron.getLatestShasForMiniAppsBranches(descriptor);
    if (updatedGitMiniApps.length === 0) {
        ern_core_1.log.info('No Changes ...');
        if (fullRegen) {
            ern_core_1.log.info('Performing regen anyway [--fullRegen]');
        }
        else {
            ern_core_1.log.info('Skipping regen. To regenerate use the --fullRegen option.');
            return;
        }
    }
    await (0, ern_orchestrator_1.syncCauldronContainer)(async () => {
        for (const updatedGitMiniApp of updatedGitMiniApps) {
            await cauldron.updateMiniAppVersionInContainer(descriptor, updatedGitMiniApp, { keepBranch: true });
        }
    }, descriptor, `Regenerate Container of ${descriptor} native application`, {
        containerVersion,
        resetCache,
        sourceMapOutput,
    });
    ern_core_1.log.info(`${descriptor} container was successfully regenerated`);
};
exports.commandHandler = commandHandler;
exports.handler = (0, lib_1.tryCatchWrap)(exports.commandHandler);
//# sourceMappingURL=regen-container.js.map