"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.commandHandler = exports.builder = exports.desc = exports.command = void 0;
const ern_core_1 = require("ern-core");
const ern_cauldron_api_1 = require("ern-cauldron-api");
const ern_orchestrator_1 = require("ern-orchestrator");
const lib_1 = require("../../../lib");
exports.command = 'miniapps <miniapps..>';
exports.desc = 'Update the version(s) of one or more MiniApp(s) in the Cauldron';
const builder = (argv) => {
    return argv
        .option('containerVersion', {
        alias: 'v',
        describe: 'Version to use for generated container. If none provided, patch version will be bumped by default.',
        type: 'string',
    })
        .option('descriptor', {
        alias: 'd',
        describe: 'A complete native application descriptor',
        type: 'string',
    })
        .coerce('descriptor', (d) => ern_core_1.AppVersionDescriptor.fromString(d))
        .option('fullRegen', {
        describe: 'Perform complete regeneration',
        type: 'boolean',
    })
        .coerce('miniapps', (d) => d.map(ern_core_1.PackagePath.fromString))
        .option('resetCache', {
        describe: 'Indicates whether to reset the React Native cache prior to bundling',
        type: 'boolean',
    })
        .option('targetVersion', {
        describe: 'Target version to update all MiniApps to. Can only be used if `all` is used for MiniApps.',
        type: 'string',
    })
        .epilog((0, lib_1.epilog)(exports));
};
exports.builder = builder;
const commandHandler = async ({ containerVersion, descriptor, fullRegen, miniapps, resetCache, targetVersion, }) => {
    descriptor =
        descriptor ||
            (await (0, lib_1.askUserToChooseANapDescriptorFromCauldron)({
                onlyNonReleasedVersions: true,
            }));
    const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
    if (miniapps.length === 1 && miniapps[0].basePath === 'all') {
        if (!targetVersion) {
            throw new Error(`missing --targetVersion option`);
        }
        const x = await cauldron.getContainerMiniApps(descriptor);
        miniapps = x.map((p) => p.isGitPath
            ? ern_core_1.PackagePath.fromString(`${p.basePath}#${targetVersion}`)
            : ern_core_1.PackagePath.fromString(`${p.basePath}@${targetVersion}`));
        ern_core_1.log.info(`Updating all MiniApps from ${descriptor} Container to target version ${targetVersion}`);
    }
    await (0, lib_1.logErrorAndExitIfNotSatisfied)({
        isNewerContainerVersion: containerVersion
            ? {
                containerVersion,
                descriptor,
                extraErrorMessage: 'To avoid conflicts with previous versions, you can only use container version newer than the current one',
            }
            : undefined,
        isSupportedMiniAppOrJsApiImplVersion: {
            obj: miniapps,
        },
        miniAppIsInNativeApplicationVersionContainer: {
            descriptor,
            extraErrorMessage: 'If you want to add a new MiniApp(s), use -ern cauldron add miniapps- instead',
            miniApp: miniapps,
        },
        miniAppIsInNativeApplicationVersionContainerWithDifferentVersion: {
            descriptor,
            extraErrorMessage: 'It seems like you are trying to update a MiniApp to a version that is already the one in use.',
            miniApp: miniapps,
        },
        napDescriptorExistInCauldron: {
            descriptor,
            extraErrorMessage: 'This command cannot work on a non existing native application version',
        },
    });
    // Special handling for git based MiniApps
    // Indeed, if only the branch or tag a MiniApp has been updated, but the head commit SHA
    // is still the same, then we shouldn't consider the MiniApp as an updated MiniApp
    // given that it will not contain any changes at all. We should just update the branch
    // in the Cauldron, but not go through complete handling.
    const updatedMiniApps = [];
    const containerMiniApps = await cauldron.getContainerMiniApps(descriptor);
    for (const miniapp of miniapps) {
        if ((await ern_core_1.utils.isGitBranch(miniapp)) || (await ern_core_1.utils.isGitTag(miniapp))) {
            const headCommitSha = await ern_core_1.utils.getCommitShaOfGitBranchOrTag(miniapp);
            if (!containerMiniApps.some((m) => m.basePath === miniapp.basePath && m.version === headCommitSha)) {
                updatedMiniApps.push(miniapp);
            }
        }
        else {
            updatedMiniApps.push(miniapp);
        }
    }
    const cauldronCommitMessage = [
        `${miniapps.length === 1
            ? `Update ${miniapps[0]} MiniApp version in ${descriptor}`
            : `Update multiple MiniApps versions in ${descriptor}`}`,
    ];
    if (updatedMiniApps.length === 0) {
        ern_core_1.log.info('No changes to MiniApps resolved SHAs (pointing to same commit(s))');
        if (fullRegen) {
            ern_core_1.log.info('Performing regen anyway [--fullRegen]');
        }
        else {
            ern_core_1.log.info(`Skipping Container regen.
Only updating Cauldron with new MiniApps versions.
To regenerate anyway use the --fullRegen option.`);
            await cauldron.beginTransaction();
            await cauldron.syncContainerMiniApps(descriptor, miniapps);
            await cauldron.commitTransaction(cauldronCommitMessage);
            return;
        }
    }
    await (0, ern_orchestrator_1.syncCauldronContainer)(async () => {
        for (const miniApp of miniapps) {
            cauldronCommitMessage.push(`- Update ${miniApp.basePath} MiniApp version to v${miniApp.version}`);
        }
        await cauldron.syncContainerMiniApps(descriptor, miniapps);
    }, descriptor, cauldronCommitMessage, {
        containerVersion,
        resetCache,
    });
    ern_core_1.log.info(`MiniApp(s) version(s) successfully updated in ${descriptor}`);
};
exports.commandHandler = commandHandler;
exports.handler = (0, lib_1.tryCatchWrap)(exports.commandHandler);
//# sourceMappingURL=miniapps.js.map