"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.commandHandler = exports.builder = exports.desc = exports.command = void 0;
const ern_core_1 = require("ern-core");
const ern_cauldron_api_1 = require("ern-cauldron-api");
const ern_orchestrator_1 = require("ern-orchestrator");
const lib_1 = require("../../lib");
const lodash_1 = __importDefault(require("lodash"));
exports.command = 'release';
exports.desc = 'CodePush MiniApp(s) or JS API implementation(s) version(s) to a target native application version';
const builder = (argv) => {
    return argv
        .option('baseComposite', {
        describe: 'Base Composite',
        type: 'string',
    })
        .coerce('baseComposite', (d) => ern_core_1.PackagePath.fromString(d))
        .option('deploymentName', {
        describe: 'Deployment to release the update to',
        type: 'string',
    })
        .option('description', {
        alias: 'des',
        describe: 'Description of the changes made to the app with this release',
        type: 'string',
    })
        .option('descriptors', {
        alias: 'd',
        describe: 'Full native application descriptors (target native application versions for the push)',
        type: 'array',
    })
        .coerce('descriptors', (d) => d.map((t) => ern_core_1.AppVersionDescriptor.fromString(t)))
        .option('force', {
        alias: 'f',
        describe: 'Force upgrade (ignore compatibility issues -at your own risks-)',
        type: 'boolean',
    })
        .option('disableDuplicateReleaseError', {
        default: false,
        describe: 'When this flag is set, releasing a package that is identical to the latest release will produce a warning instead of an error',
        type: 'boolean',
    })
        .option('jsApiImpls', {
        describe: 'One or more JS API implementation to CodePush',
        type: 'array',
    })
        .option('mandatory', {
        alias: 'm',
        default: false,
        describe: 'Specifies whether this release should be considered mandatory',
        type: 'boolean',
    })
        .option('miniapps', {
        describe: 'One or more MiniApp to CodePush',
        type: 'array',
    })
        .option('rollout', {
        alias: 'r',
        default: 100,
        describe: 'Percentage of users this release should be immediately available to',
        type: 'number',
    })
        .option('semVerDescriptor', {
        describe: 'A native application descriptor using a semver expression for the version',
    })
        .option('skipConfirmation', {
        alias: 's',
        describe: 'Skip confirmation prompts',
        type: 'boolean',
    })
        .option('sourceMapOutput', {
        describe: 'Path to source map file to generate for this codepush',
        type: 'string',
    })
        .option('targetBinaryVersion', {
        alias: 't',
        describe: 'Semver expression that specifies the binary app version(s) this release is targeting',
        type: 'string',
    })
        .epilog((0, lib_1.epilog)(exports));
};
exports.builder = builder;
const commandHandler = async ({ baseComposite, deploymentName, description, descriptors = [], disableDuplicateReleaseError, force, jsApiImpls = [], mandatory, miniapps = [], rollout, skipConfirmation, semVerDescriptor, sourceMapOutput, targetBinaryVersion, }) => {
    if (miniapps.length === 0 && jsApiImpls.length === 0) {
        throw new Error('You need to provide at least one MiniApp or one JS API implementation version to CodePush');
    }
    await (0, lib_1.logErrorAndExitIfNotSatisfied)({
        checkIfCodePushOptionsAreValid: {
            descriptors,
            semVerDescriptor,
            targetBinaryVersion,
        },
    });
    if (descriptors.length > 0) {
        // User provided one or more descriptor(s)
        await (0, lib_1.logErrorAndExitIfNotSatisfied)({
            napDescriptorExistInCauldron: {
                descriptor: descriptors,
                extraErrorMessage: 'You cannot CodePush to a non existing native application version.',
            },
            sameNativeApplicationAndPlatform: {
                descriptors,
                extraErrorMessage: 'You can only pass descriptors that match the same native application and version',
            },
        });
    }
    else if (descriptors.length === 0 && !semVerDescriptor) {
        // User provided no descriptors, nor a semver descriptor
        descriptors = await (0, lib_1.askUserToChooseOneOrMoreNapDescriptorFromCauldron)({
            onlyReleasedVersions: true,
        });
    }
    else if (semVerDescriptor) {
        // User provided a semver Descriptor
        const semVerNapDescriptor = ern_core_1.AppVersionDescriptor.fromString(semVerDescriptor);
        const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
        descriptors = await cauldron.getDescriptorsMatchingSemVerDescriptor(semVerNapDescriptor);
        if (descriptors.length === 0) {
            throw new Error(`No versions matching ${semVerDescriptor} were found`);
        }
        else {
            ern_core_1.log.info('CodePush release will target the following native application descriptors :');
            for (const descriptor of descriptors) {
                ern_core_1.log.info(`- ${descriptor}`);
            }
            if (!skipConfirmation) {
                const userConfirmedVersions = await (0, lib_1.askUserConfirmation)('Do you want to proceed ?');
                if (!userConfirmedVersions) {
                    throw new Error('Aborting command execution');
                }
            }
        }
    }
    await (0, lib_1.logErrorAndExitIfNotSatisfied)({
        noFileSystemPath: {
            extraErrorMessage: 'You cannot provide dependencies using git or file scheme for this command. Only the form miniapp@version is allowed.',
            obj: [...miniapps, ...jsApiImpls],
        },
    });
    const miniAppsPackages = lodash_1.default.map(miniapps, ern_core_1.PackagePath.fromString);
    const jsApiImplsPackages = lodash_1.default.map(jsApiImpls, ern_core_1.PackagePath.fromString);
    const packages = [...miniAppsPackages, ...jsApiImplsPackages];
    for (const pkg of packages) {
        if (pkg.isGitPath && (await ern_core_1.utils.isGitBranch(pkg))) {
            throw new Error('You cannot code push packages from a git branch. Only SHA or TAGs are supported.');
        }
    }
    if (!deploymentName) {
        deploymentName = await (0, lib_1.askUserForCodePushDeploymentName)(descriptors[0]);
    }
    for (const descriptor of descriptors) {
        const pathToYarnLock = await getPathToYarnLock(descriptor, deploymentName);
        await (0, ern_orchestrator_1.performCodePushOtaUpdate)(descriptor, deploymentName, miniAppsPackages, jsApiImplsPackages, {
            baseComposite,
            codePushIsMandatoryRelease: mandatory,
            codePushRolloutPercentage: rollout,
            description,
            disableDuplicateReleaseError,
            force,
            pathToYarnLock: pathToYarnLock || undefined,
            sourceMapOutput,
            targetBinaryVersion,
        });
    }
    ern_core_1.log.info(`Successfully released`);
};
exports.commandHandler = commandHandler;
async function getPathToYarnLock(napDescriptor, deploymentName) {
    const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)();
    if (!cauldron) {
        throw new Error('[getPathToYarnLock] No active Cauldron');
    }
    let pathToYarnLock = await cauldron.getPathToYarnLock(napDescriptor, deploymentName);
    if (!pathToYarnLock) {
        pathToYarnLock = await cauldron.getPathToYarnLock(napDescriptor, 'container');
    }
    return pathToYarnLock;
}
exports.handler = (0, lib_1.tryCatchWrap)(exports.commandHandler);
//# sourceMappingURL=release.js.map