"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.commandHandler = exports.builder = exports.desc = exports.command = void 0;
const ern_core_1 = require("ern-core");
const ern_api_impl_gen_1 = require("ern-api-impl-gen");
const lib_1 = require("../lib");
const path_1 = __importDefault(require("path"));
const untildify_1 = __importDefault(require("untildify"));
exports.command = 'create-api-impl <apiName> [apiImplName]';
exports.desc = 'Commands to generate API implementation skeleton.';
const builder = (argv) => {
    return argv
        .option('force', {
        alias: 'f',
        describe: 'Forces a project creation even if an implementation already present inside the output location',
        type: 'boolean',
    })
        .option('hasConfig', {
        describe: 'Indicates if this api implementation requires some config during initialization.\nThis option will be stored and reused during container generation to enforce config initialization',
        type: 'boolean',
    })
        .option('jsOnly', {
        alias: 'j',
        describe: 'Generate js project with proper dependencies (Implementation of the API has to be written in js)',
        type: 'boolean',
    })
        .option('manifestId', {
        describe: 'Id of the Manifest entry to use',
        type: 'string',
    })
        .option('nativeOnly', {
        alias: 'n',
        describe: 'Generate native projects with proper dependencies (Implementation of the API has to be written in native)',
        type: 'boolean',
    })
        .option('packageName', {
        alias: 'p',
        describe: 'Name to use for the apiImpl npm package',
    })
        .option('scope', {
        alias: 's',
        describe: 'Scope to use for the apiImpl npm package',
    })
        .option('skipNpmCheck', {
        describe: 'Skip the check ensuring package does not already exists in npm registry',
        type: 'boolean',
    })
        .option('outputDirectory', {
        alias: 'o',
        describe: 'Path to output directory',
    })
        .coerce('outputDirectory', (p) => (0, untildify_1.default)(p))
        .epilog((0, lib_1.epilog)(exports));
};
exports.builder = builder;
const WORKING_DIRECTORY = path_1.default.join(ern_core_1.Platform.rootDirectory, 'api-impl-gen');
const PLUGIN_DIRECTORY = path_1.default.join(WORKING_DIRECTORY, 'plugins');
const commandHandler = async ({ apiName, apiImplName, force, hasConfig, jsOnly, manifestId, nativeOnly, packageName, scope, skipNpmCheck, outputDirectory, }) => {
    const apiDep = ern_core_1.PackagePath.fromString(apiName);
    // pre conditions
    await (0, lib_1.logErrorAndExitIfNotSatisfied)({
        noGitOrFilesystemPath: {
            obj: apiName,
        },
        publishedToNpm: {
            extraErrorMessage: `Couldn't find package ${apiName} to generate the api implementation`,
            obj: apiName,
        },
    });
    if (apiImplName) {
        await (0, lib_1.logErrorAndExitIfNotSatisfied)({
            isValidElectrodeNativeModuleName: {
                name: apiImplName,
            },
        });
    }
    if (manifestId) {
        await (0, lib_1.logErrorAndExitIfNotSatisfied)({
            manifestIdExists: {
                id: manifestId,
            },
        });
    }
    const reactNativeVersion = await ern_core_1.utils.reactNativeManifestVersion({
        manifestId,
    });
    if (!reactNativeVersion) {
        throw new Error('React Native version is not defined in Manifest. This should not happen!');
    }
    if (jsOnly && nativeOnly) {
        ern_core_1.log.warn('Looks like both js and native are selected, should be only one');
        nativeOnly = (await (0, lib_1.askUserToSelectAnEnvironment)()) !== 'js';
        jsOnly = !nativeOnly;
    }
    if (!jsOnly && !nativeOnly) {
        nativeOnly = (await (0, lib_1.askUserToSelectAnEnvironment)()) !== 'js';
        jsOnly = !nativeOnly;
    }
    const moduleType = nativeOnly
        ? ern_core_1.ModuleTypes.NATIVE_API_IMPL
        : ern_core_1.ModuleTypes.JS_API_IMPL;
    if (apiImplName && !(0, ern_core_1.validateModuleName)(apiImplName, moduleType)) {
        apiImplName = await (0, lib_1.promptUserToUseSuggestedModuleName)(apiImplName, moduleType);
    }
    // Must conform to definition of ElectrodeNativeModuleName
    if (!apiImplName) {
        const simpleName = apiName.replace(/^@(.+)\/|-api$/g, '');
        apiImplName = `${simpleName}-${ern_core_1.utils.getModuleSuffix(moduleType)}`;
    }
    // If no package name is specified get default name from apiImplName
    if (!packageName) {
        packageName = ern_core_1.utils.getDefaultPackageNameForModule(apiImplName, moduleType);
    }
    // Check if packageName is valid
    await (0, lib_1.logErrorAndExitIfNotSatisfied)({
        isValidNpmPackageName: {
            name: packageName,
        },
    });
    // Skip npm check
    if (!skipNpmCheck && !(await (0, lib_1.performPkgNameConflictCheck)(packageName))) {
        throw new Error(`Aborting command `);
    }
    ern_core_1.log.info(`Generating API implementation for ${apiName}`);
    await (0, ern_api_impl_gen_1.generateApiImpl)({
        apiDependency: apiDep,
        apiImplName,
        forceGenerate: force,
        hasConfig,
        nativeOnly,
        outputDirectory,
        packageName,
        paths: {
            apiImplHull: path_1.default.join(ern_core_1.Platform.currentPlatformVersionPath, 'ern-api-impl-gen', 'hull'),
            outDirectory: '',
        },
        reactNativeVersion,
        scope,
    });
    ern_core_1.log.info('Success');
};
exports.commandHandler = commandHandler;
exports.handler = (0, lib_1.tryCatchWrap)(exports.commandHandler);
//# sourceMappingURL=create-api-impl.js.map