"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.commandHandler = exports.builder = exports.desc = exports.command = void 0;
const fs_extra_1 = __importDefault(require("fs-extra"));
const ern_api_gen_1 = require("ern-api-gen");
const ern_core_1 = require("ern-core");
const lib_1 = require("../lib");
exports.command = 'create-api <apiName>';
exports.desc = 'Create a new api';
const builder = (argv) => {
    return argv
        .option('apiAuthor', {
        alias: 'u',
        describe: 'Author of library',
    })
        .option('apiVersion', {
        alias: 'a',
        describe: 'Initial npm version',
    })
        .option('manifestId', {
        describe: 'Id of the manifest entry to use',
        type: 'string',
    })
        .option('packageName', {
        alias: 'p',
        describe: 'Name to use for the api npm package',
    })
        .option('schemaPath', {
        alias: 'm',
        describe: 'Path to pre-existing schema(swagger)',
    })
        .option('scope', {
        alias: 's',
        describe: 'Scope to use for the api npm package',
    })
        .option('skipNpmCheck', {
        describe: 'Skip the check ensuring package does not already exists in npm registry',
        type: 'boolean',
    })
        .epilog((0, lib_1.epilog)(exports));
};
exports.builder = builder;
const commandHandler = async ({ apiAuthor, apiName, apiVersion, manifestId, packageName, schemaPath, scope, skipNpmCheck, }) => {
    await (0, lib_1.logErrorAndExitIfNotSatisfied)({
        isValidElectrodeNativeModuleName: {
            name: apiName,
        },
    });
    if (manifestId) {
        await (0, lib_1.logErrorAndExitIfNotSatisfied)({
            manifestIdExists: {
                id: manifestId,
            },
        });
    }
    if (schemaPath && !(await fs_extra_1.default.pathExists(schemaPath))) {
        throw new Error(`Cannot resolve path to ${schemaPath}`);
    }
    if (!(0, ern_core_1.validateModuleName)(apiName, ern_core_1.ModuleTypes.API)) {
        apiName = await (0, lib_1.promptUserToUseSuggestedModuleName)(apiName, ern_core_1.ModuleTypes.API);
    }
    if (!packageName) {
        packageName = ern_core_1.utils.getDefaultPackageNameForModule(apiName, ern_core_1.ModuleTypes.API);
    }
    await (0, lib_1.logErrorAndExitIfNotSatisfied)({
        isValidNpmPackageName: {
            name: packageName,
        },
    });
    if (!skipNpmCheck && !(await (0, lib_1.performPkgNameConflictCheck)(packageName))) {
        throw new Error('Aborting command');
    }
    const bridgeDep = await ern_core_1.manifest.getNativeDependency(ern_core_1.PackagePath.fromString('react-native-electrode-bridge'), { manifestId });
    if (!bridgeDep) {
        throw new Error('react-native-electrode-bridge not found in manifest. cannot infer version to use');
    }
    const reactNative = await ern_core_1.manifest.getNativeDependency(ern_core_1.PackagePath.fromString('react-native'), { manifestId });
    if (!reactNative) {
        throw new Error('react-native not found in manifest. cannot infer version to use');
    }
    ern_core_1.log.info(`Generating ${apiName}`);
    await ern_api_gen_1.ApiGen.generateApi({
        apiAuthor,
        apiSchemaPath: schemaPath,
        apiVersion,
        bridgeVersion: `${bridgeDep.version || ''}`,
        name: apiName,
        npmScope: scope,
        packageName,
        reactNativeVersion: reactNative.version,
    });
    ern_core_1.log.info('Success');
};
exports.commandHandler = commandHandler;
exports.handler = (0, lib_1.tryCatchWrap)(exports.commandHandler);
//# sourceMappingURL=create-api.js.map