"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.commandHandler = exports.builder = exports.desc = exports.command = void 0;
const ern_composite_gen_1 = require("ern-composite-gen");
const ern_core_1 = require("ern-core");
const ern_cauldron_api_1 = require("ern-cauldron-api");
const lib_1 = require("../lib");
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const untildify_1 = __importDefault(require("untildify"));
exports.command = 'create-composite';
exports.desc = 'Create a JS composite project locally';
const builder = (argv) => {
    return argv
        .option('baseComposite', {
        describe: 'Base Composite',
        type: 'string',
    })
        .coerce('baseComposite', (d) => ern_core_1.PackagePath.fromString(d))
        .option('descriptor', {
        alias: 'd',
        describe: 'Full native application descriptor',
        type: 'string',
    })
        .coerce('descriptor', (d) => ern_core_1.AppVersionDescriptor.fromString(d))
        .option('extraJsDependencies', {
        alias: 'e',
        describe: 'Additional JS dependency(ies)',
        type: 'array',
    })
        .coerce('extraJsDependencies', (d) => d.map(ern_core_1.PackagePath.fromString))
        .option('fromGitBranches', {
        describe: 'Favor MiniApp(s) branches',
        type: 'boolean',
    })
        .option('jsApiImpls', {
        describe: 'One or more JS API implementation(s)',
        type: 'array',
    })
        .coerce('jsApiImpls', (d) => d.map(ern_core_1.PackagePath.fromString))
        .option('miniapps', {
        alias: 'm',
        describe: 'One or more MiniApp(s)',
        type: 'array',
    })
        .coerce('miniapps', (d) => d.map(ern_core_1.PackagePath.fromString))
        .option('outDir', {
        alias: 'out',
        describe: 'Output directory',
        type: 'string',
    })
        .coerce('outDir', (p) => (0, untildify_1.default)(p))
        .epilog((0, lib_1.epilog)(exports));
};
exports.builder = builder;
const commandHandler = async ({ baseComposite, descriptor, extraJsDependencies, fromGitBranches, jsApiImpls, miniapps, outDir, } = {}) => {
    if (outDir && (await fs_extra_1.default.pathExists(outDir))) {
        if ((await fs_extra_1.default.readdir(outDir)).length > 0) {
            throw new Error(`${outDir} directory exists and is not empty.
Output directory should either not exist (it will be created) or should be empty.`);
        }
    }
    outDir = outDir || path_1.default.join(ern_core_1.Platform.rootDirectory, 'miniAppsComposite');
    const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)({ throwIfNoActiveCauldron: false });
    if (!cauldron && !miniapps) {
        throw new Error("A Cauldron must be active if you don't explicitly provide miniapps");
    }
    // Full native application selector was not provided.
    // Ask the user to select a completeNapDescriptor from a list
    // containing all the native applications versions in the cauldron
    // Not needed if miniapps are directly provided
    if (!descriptor && !miniapps) {
        descriptor = await (0, lib_1.askUserToChooseANapDescriptorFromCauldron)();
    }
    let pathToYarnLock;
    let resolutions;
    let metroExtraNodeModules;
    if (descriptor && cauldron) {
        await (0, lib_1.logErrorAndExitIfNotSatisfied)({
            napDescriptorExistInCauldron: {
                descriptor,
                extraErrorMessage: 'You cannot create a composite for a non-existing native application version.',
            },
        });
        miniapps = await cauldron.getContainerMiniApps(descriptor, {
            favorGitBranches: !!fromGitBranches,
        });
        jsApiImpls = await cauldron.getContainerJsApiImpls(descriptor);
        const containerGenConfig = await cauldron.getContainerGeneratorConfig(descriptor);
        if (!containerGenConfig || !containerGenConfig.bypassYarnLock) {
            pathToYarnLock = await cauldron.getPathToYarnLock(descriptor, 'container');
        }
        else {
            ern_core_1.log.debug('Bypassing yarn.lock usage as bypassYarnLock flag is set in config');
        }
        const compositeGenConfig = await cauldron.getCompositeGeneratorConfig(descriptor);
        baseComposite =
            baseComposite ||
                ((compositeGenConfig === null || compositeGenConfig === void 0 ? void 0 : compositeGenConfig.baseComposite) &&
                    ern_core_1.PackagePath.fromString(compositeGenConfig.baseComposite));
        resolutions = compositeGenConfig && compositeGenConfig.resolutions;
        metroExtraNodeModules =
            compositeGenConfig && compositeGenConfig.metroExtraNodeModules;
    }
    await ern_core_1.kax.task('Generating Composite').run((0, ern_composite_gen_1.generateComposite)({
        baseComposite,
        extraJsDependencies,
        jsApiImplDependencies: jsApiImpls,
        metroExtraNodeModules,
        miniApps: miniapps,
        outDir,
        pathToYarnLock,
        resolutions,
    }));
    ern_core_1.log.info(`Composite successfully generated in ${outDir}`);
};
exports.commandHandler = commandHandler;
exports.handler = (0, lib_1.tryCatchWrap)(exports.commandHandler);
//# sourceMappingURL=create-composite.js.map