"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.commandHandler = exports.builder = exports.desc = exports.command = void 0;
const ern_core_1 = require("ern-core");
const ern_cauldron_api_1 = require("ern-cauldron-api");
const ern_orchestrator_1 = require("ern-orchestrator");
const lib_1 = require("../lib");
const fs_extra_1 = __importDefault(require("fs-extra"));
const untildify_1 = __importDefault(require("untildify"));
exports.command = 'create-container';
exports.desc = 'Create a Container locally';
const builder = (argv) => {
    return argv
        .option('baseComposite', {
        describe: 'Base Composite',
        type: 'string',
    })
        .coerce('baseComposite', (d) => ern_core_1.PackagePath.fromString(d))
        .option('compositeDir', {
        describe: 'Directory in which to generate the Composite',
        type: 'string',
    })
        .coerce('compositeDir', (p) => (0, untildify_1.default)(p))
        .option('descriptor', {
        alias: 'd',
        describe: 'Full native application descriptor',
        type: 'string',
    })
        .coerce('descriptor', (d) => ern_core_1.AppVersionDescriptor.fromString(d))
        .option('extra', {
        alias: 'e',
        describe: 'Optional extra run configuration (json string or local/cauldron path to config file)',
        type: 'string',
    })
        .option('devJsBundle', {
        describe: 'Generate a development JavaScript bundle rather than a production one',
        type: 'boolean',
    })
        .option('fromGitBranches', {
        describe: 'Create Container based on MiniApps branches rather than current MiniApps SHAs',
        type: 'boolean',
    })
        .option('ignoreRnpmAssets', {
        describe: 'Ignore rnpm assets from the MiniApps',
        type: 'boolean',
    })
        .option('jsApiImpls', {
        describe: 'A list of one or more JS API implementation',
        type: 'array',
    })
        .coerce('jsApiImpls', (d) => d.map(ern_core_1.PackagePath.fromString))
        .option('miniapps', {
        alias: 'm',
        describe: 'A list of one or more miniapps',
        type: 'array',
    })
        .coerce('miniapps', (d) => d.map(ern_core_1.PackagePath.fromString))
        .option('platform', {
        alias: 'p',
        choices: ['android', 'ios', undefined],
        describe: 'The platform for which to generate the container',
        type: 'string',
    })
        .option('resetCache', {
        describe: 'Indicates whether to reset the React Native cache prior to bundling',
        type: 'boolean',
    })
        .option('skipInstall', {
        default: !(process.platform === 'darwin'),
        describe: 'Skip `yarn install` and `pod install` after generating the container (iOS RN>=0.61 only)',
        type: 'boolean',
    })
        .option('sourceMapOutput', {
        describe: 'Path to source map file to generate for this container bundle',
        type: 'string',
    })
        .coerce('sourceMapOutput', (p) => (0, untildify_1.default)(p))
        .option('outDir', {
        alias: 'out',
        describe: 'Directory to output the generated container to',
        type: 'string',
    })
        .coerce('outDir', (p) => (0, untildify_1.default)(p))
        .epilog((0, lib_1.epilog)(exports));
};
exports.builder = builder;
const commandHandler = async ({ baseComposite, compositeDir, descriptor, devJsBundle, extra, fromGitBranches, ignoreRnpmAssets, jsApiImpls, miniapps, outDir, platform, resetCache, skipInstall, sourceMapOutput, } = {}) => {
    var _a, _b;
    if (outDir && (await fs_extra_1.default.pathExists(outDir))) {
        if ((await fs_extra_1.default.readdir(outDir)).length > 0) {
            throw new Error(`${outDir} directory exists and is not empty.
Output directory should either not exist (it will be created) or should be empty.`);
        }
    }
    compositeDir = compositeDir || (0, ern_core_1.createTmpDir)();
    const cauldron = await (0, ern_cauldron_api_1.getActiveCauldron)({ throwIfNoActiveCauldron: false });
    if (!cauldron && !miniapps) {
        throw new Error("A Cauldron must be active, if you don't explicitly provide miniapps");
    }
    let extraObj;
    try {
        extraObj = (extra && (await (0, ern_orchestrator_1.parseJsonFromStringOrFile)(extra))) || {};
    }
    catch (e) {
        throw new Error('(--extra/-e option): Invalid input');
    }
    // Full native application selector was not provided.
    // Ask the user to select a completeNapDescriptor from a list
    // containing all the native applications versions in the cauldron
    // Not needed if miniapps are directly provided
    if (!descriptor && !miniapps) {
        descriptor = await (0, lib_1.askUserToChooseANapDescriptorFromCauldron)({
            onlyNonReleasedVersions: true,
        });
    }
    if (descriptor) {
        await (0, lib_1.logErrorAndExitIfNotSatisfied)({
            napDescriptorExistInCauldron: {
                descriptor,
                extraErrorMessage: 'You cannot create a container for a non-existing native application version.',
            },
        });
    }
    const compositeGenConfig = await (cauldron === null || cauldron === void 0 ? void 0 : cauldron.getCompositeGeneratorConfig(descriptor));
    baseComposite =
        baseComposite ||
            ((compositeGenConfig === null || compositeGenConfig === void 0 ? void 0 : compositeGenConfig.baseComposite) &&
                ern_core_1.PackagePath.fromString(compositeGenConfig.baseComposite));
    if (!descriptor && miniapps) {
        platform = platform || (await (0, lib_1.askUserToSelectAPlatform)());
        if (platform === 'ios') {
            extraObj.iosConfig = extraObj.iosConfig || {};
            extraObj.iosConfig.skipInstall =
                extraObj.iosConfig.skipInstall || skipInstall;
        }
        const composite = await ern_core_1.kax.task('Generating Composite locally').run((0, ern_orchestrator_1.runLocalCompositeGen)({
            baseComposite,
            jsApiImpls,
            metroExtraNodeModules: compositeGenConfig === null || compositeGenConfig === void 0 ? void 0 : compositeGenConfig.metroExtraNodeModules,
            miniApps: miniapps,
            outDir: compositeDir,
            resolutions: (_b = (_a = extraObj === null || extraObj === void 0 ? void 0 : extraObj.compositeGenerator) === null || _a === void 0 ? void 0 : _a.resolutions) !== null && _b !== void 0 ? _b : compositeGenConfig === null || compositeGenConfig === void 0 ? void 0 : compositeGenConfig.resolutions,
        }));
        outDir = outDir || ern_core_1.Platform.getContainerGenOutDirectory(platform);
        await ern_core_1.kax.task('Generating Container locally').run((0, ern_orchestrator_1.runLocalContainerGen)(platform, composite, {
            devJsBundle,
            extra: extraObj,
            ignoreRnpmAssets,
            outDir,
            resetCache,
            sourceMapOutput,
        }));
    }
    else if (descriptor) {
        const composite = await ern_core_1.kax.task('Generating Composite from Cauldron').run((0, ern_orchestrator_1.runCauldronCompositeGen)(descriptor, {
            baseComposite,
            favorGitBranches: !!fromGitBranches,
            outDir: compositeDir,
        }));
        outDir =
            outDir || ern_core_1.Platform.getContainerGenOutDirectory(descriptor.platform);
        await ern_core_1.kax.task('Generating Container from Cauldron').run((0, ern_orchestrator_1.runCauldronContainerGen)(descriptor, composite, {
            devJsBundle,
            outDir,
            resetCache,
            sourceMapOutput,
        }));
    }
    ern_core_1.log.info(`Container successfully generated in ${outDir}\nComposite generated in ${compositeDir}`);
};
exports.commandHandler = commandHandler;
exports.handler = (0, lib_1.tryCatchWrap)(exports.commandHandler);
//# sourceMappingURL=create-container.js.map