"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.commandHandler = exports.builder = exports.desc = exports.command = void 0;
const ern_core_1 = require("ern-core");
const lib_1 = require("../lib");
const chalk_1 = __importDefault(require("chalk"));
exports.command = 'create-miniapp <appName>';
exports.desc = 'Create a new ern application(miniapp)';
const builder = (argv) => {
    return argv
        .option('language', {
        choices: ['JavaScript', 'TypeScript', undefined],
        deprecated: 'use --template directly',
        describe: 'Language to use for this MiniApp',
        type: 'string',
    })
        .option('packageName', {
        alias: 'p',
        describe: 'Name to use for the MiniApp npm package',
    })
        .option('platformVersion', {
        alias: 'v',
        describe: 'Overrides current platform version in use with this platform version',
        type: 'string',
    })
        .option('manifestId', {
        describe: 'Id of the Manifest entry to use to create this MiniApp',
        type: 'string',
    })
        .option('npm', {
        describe: 'Use npm instead of Yarn to install dependencies',
        type: 'boolean',
    })
        .option('scope', {
        alias: 's',
        describe: 'Scope to use for the MiniApp npm package',
    })
        .option('skipInstall', {
        describe: 'Skip the installation of dependencies after project creation',
        type: 'boolean',
    })
        .option('skipNpmCheck', {
        deprecated: 'use "npm info" to check manually',
        describe: 'Skip the check ensuring package does not already exists in npm registry',
        type: 'boolean',
    })
        .option('packageManager', {
        choices: ['npm', 'yarn', undefined],
        deprecated: 'Yarn is the default, use --npm to override',
        describe: 'Package manager to use for this MiniApp',
        type: 'string',
    })
        .option('template', {
        describe: 'Template to use to create the MiniApp',
        type: 'string',
    })
        .epilog((0, lib_1.epilog)(exports));
};
exports.builder = builder;
const commandHandler = async ({ appName, language, manifestId, npm, packageManager, packageName, platformVersion, scope, skipInstall, skipNpmCheck, template, }) => {
    await (0, lib_1.logErrorAndExitIfNotSatisfied)({
        isValidElectrodeNativeModuleName: {
            name: appName,
        },
    });
    if (language) {
        ern_core_1.log.warn('Deprecated: --language. Use --template, or omit to use default.');
    }
    if (packageManager === 'yarn') {
        ern_core_1.log.warn('Deprecated: --packageManager. Yarn is the default.');
    }
    else if (packageManager === 'npm') {
        ern_core_1.log.warn('Deprecated: --packageManager. Use --npm to override default usage of Yarn.');
    }
    if (manifestId) {
        await (0, lib_1.logErrorAndExitIfNotSatisfied)({
            manifestIdExists: {
                id: manifestId,
            },
        });
    }
    if (!(0, ern_core_1.validateModuleName)(appName, ern_core_1.ModuleTypes.MINIAPP)) {
        appName = await (0, lib_1.promptUserToUseSuggestedModuleName)(appName, ern_core_1.ModuleTypes.MINIAPP);
    }
    if (!packageName) {
        packageName = ern_core_1.utils.getDefaultPackageNameForModule(appName, ern_core_1.ModuleTypes.MINIAPP);
    }
    if (packageManager === 'npm' || npm) {
        packageManager = 'npm';
    }
    else {
        packageManager = 'yarn';
    }
    await (0, lib_1.logErrorAndExitIfNotSatisfied)({
        isValidNpmPackageName: {
            name: packageName,
        },
    });
    if (skipNpmCheck) {
        ern_core_1.log.warn('Deprecated: --skipNpmCheck. create-miniapp no longer checks the npm' +
            'registry. Use "npm info" to check manually.');
    }
    await ern_core_1.kax.task('Creating MiniApp').run(ern_core_1.MiniApp.create(appName, packageName, {
        language,
        manifestId,
        packageManager,
        platformVersion: platformVersion && platformVersion.replace('v', ''),
        scope,
        skipInstall,
        template,
    }));
    ern_core_1.log.info(`${appName} MiniApp was successfully created !`);
    ern_core_1.log.info(`================================================`);
    ern_core_1.log.info(chalk_1.default.bold.white('To run your MiniApp on Android :'));
    ern_core_1.log.info(chalk_1.default.white(`    > cd ${appName}`));
    ern_core_1.log.info(chalk_1.default.white(`followed by :`));
    ern_core_1.log.info(chalk_1.default.white(`    > ern run-android`));
    ern_core_1.log.info(chalk_1.default.bold.white('To run your MiniApp on iOS :'));
    ern_core_1.log.info(chalk_1.default.white(`    > cd ${appName}`));
    ern_core_1.log.info(chalk_1.default.white(`followed by :`));
    ern_core_1.log.info(chalk_1.default.white(`    > ern run-ios`));
    ern_core_1.log.info(`================================================`);
};
exports.commandHandler = commandHandler;
exports.handler = (0, lib_1.tryCatchWrap)(exports.commandHandler);
//# sourceMappingURL=create-miniapp.js.map