"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.builder = exports.desc = exports.command = void 0;
const ern_api_impl_gen_1 = require("ern-api-impl-gen");
const ern_core_1 = require("ern-core");
const lib_1 = require("../lib");
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const semver_1 = __importDefault(require("semver"));
exports.command = 'regen-api-impl';
exports.desc = 'Regenerates an existing api implementation for a newer version of the api';
const builder = (argv) => {
    return argv
        .option('apiVersion', {
        alias: 'v',
        describe: 'a specific version of the api for which an implementation needs to be generated. \n The version should be higher than the version for which an implementation is already generated',
    })
        .option('manifestId', {
        describe: 'Id of the Manifest entry to use',
        type: 'string',
    })
        .epilog((0, lib_1.epilog)(exports));
};
exports.builder = builder;
const ERROR_MSG_NOT_IN_IMPL = 'api-regen-impl can only be run inside an implementation project directory. \n Looks like it is ran elsewhere';
const WORKING_DIRECTORY = path_1.default.join(ern_core_1.Platform.rootDirectory, `api-impl-gen`);
const PLUGIN_DIRECTORY = path_1.default.join(WORKING_DIRECTORY, 'plugins');
// TOO MUCH LOGIC IN THE COMMAND ITSELF
// TO REFACTOR TO EXTRACT LOGIC OUT OF THE COMMAND FOR REUSABILITY
const handler = async ({ apiVersion, manifestId, }) => {
    try {
        if (manifestId) {
            await (0, lib_1.logErrorAndExitIfNotSatisfied)({
                manifestIdExists: {
                    id: manifestId,
                },
            });
        }
        const apiImplPackage = await readPackageJson();
        let api = await getApi(apiImplPackage);
        const currentApiVersion = api.version;
        if (!currentApiVersion) {
            throw new Error('no API version. This should not happen.');
        }
        api = ern_core_1.PackagePath.fromString(`${api.name}${apiVersion ? `@${apiVersion}` : ''}`);
        ern_core_1.log.info(`regenerating api implementation for ${api.toString()}`);
        const reactNativeVersion = await ern_core_1.utils.reactNativeManifestVersion({
            manifestId,
        });
        if (!reactNativeVersion) {
            throw new Error('React Native version is not defined in Manifest. This sould not happen !');
        }
        ern_core_1.log.debug(`Will generate api implementation using react native version: ${reactNativeVersion}`);
        await validatePackage(api);
        await performVersionCheck(api, currentApiVersion, apiVersion);
        await (0, ern_api_impl_gen_1.regenerateApiImpl)({
            api,
            paths: {
                apiImplHull: path_1.default.join(ern_core_1.Platform.currentPlatformVersionPath, 'ern-api-impl-gen', 'hull'),
                outDirectory: process.cwd(),
            },
            platforms: getPlatforms(),
            reactNativeVersion,
        });
        ern_core_1.log.info('Successfully regenerated api implementation!');
    }
    catch (e) {
        ern_core_1.utils.logErrorAndExitProcess(e);
    }
    async function readPackageJson() {
        const packageJsonPath = path_1.default.join(process.cwd(), 'package.json');
        ern_core_1.log.debug(`Reading package json: ${packageJsonPath}`);
        if (!(await fs_extra_1.default.pathExists(packageJsonPath))) {
            ern_core_1.log.error(`${packageJsonPath} not found`);
            throw new Error(ERROR_MSG_NOT_IN_IMPL);
        }
        const apiImplPackage = await fs_extra_1.default.readJson(packageJsonPath);
        if (!apiImplPackage.ern ||
            (apiImplPackage.ern.moduleType !== ern_core_1.ModuleTypes.NATIVE_API_IMPL &&
                apiImplPackage.ern.moduleType !== ern_core_1.ModuleTypes.JS_API_IMPL)) {
            ern_core_1.log.error('Not an api implementation');
            throw new Error(ERROR_MSG_NOT_IN_IMPL);
        }
        return apiImplPackage;
    }
    async function getApi(apiImplPackage) {
        for (const depKey of Object.keys(apiImplPackage.dependencies)) {
            if (await ern_core_1.utils.isDependencyApi(ern_core_1.PackagePath.fromString(depKey))) {
                // TODO: THis is by assuming that this is the only api dependency inside this implemenation.
                // TODO: This may not be right all the time as an api implementor can add more other apis as dependencies. Logic needs to be revisited.
                return ern_core_1.PackagePath.fromString(`${depKey}@${apiImplPackage.dependencies[depKey]}`);
            }
        }
        throw new Error('Unable to identify the api for this implementation');
    }
    async function validatePackage(api) {
        if (!(await ern_core_1.utils.isPublishedToNpm(api.toString()))) {
            throw new Error(`${api.toString()}: Package not found in npm, please make sure this version of the api is published to npm.`);
        }
    }
    async function performVersionCheck(api, currentApiVersion, version) {
        ern_core_1.log.debug('Performing version check before regenerating the code.');
        if (!version) {
            const latestReleasedPackageJson = await ern_core_1.yarn.info(api);
            version = latestReleasedPackageJson.version;
        }
        if (version && semver_1.default.lte(version, currentApiVersion)) {
            ern_core_1.log.warn(`You are generating an api implementation for an api version(${version}) that is less than or equal to the current one(${currentApiVersion}). `);
        }
        else {
            ern_core_1.log.info(`Regenerating api implementation for apiVersion:${version}, current apiVersion:${currentApiVersion}`);
        }
    }
    function getPlatforms() {
        const nativeDirectory = path_1.default.join(process.cwd(), 'android');
        return fs_extra_1.default.pathExistsSync(nativeDirectory) ? ['android', 'ios'] : ['js'];
    }
};
exports.handler = handler;
//# sourceMappingURL=regen-api-impl.js.map