"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const child_process_1 = require("child_process");
const fs_extra_1 = __importDefault(require("fs-extra"));
const ern_cauldron_api_1 = require("ern-cauldron-api");
const ern_core_1 = require("ern-core");
const kax_1 = require("kax");
const path_1 = __importDefault(require("path"));
const yargs_1 = __importDefault(require("yargs"));
const semver_1 = __importDefault(require("semver"));
// ==============================================================================
// Geeky eye candy
// ==============================================================================
function showBanner() {
    console.log(chalk_1.default.yellow(' ___ _        _               _      ') +
        chalk_1.default.green(' _  _      _   _         '));
    console.log(chalk_1.default.yellow('| __| |___ __| |_ _ _ ___  __| |___  ') +
        chalk_1.default.green('| \\| |__ _| |_(_)_ _____ '));
    console.log(chalk_1.default.yellow("| _|| / -_) _|  _| '_/ _ \\/ _` / -_) ") +
        chalk_1.default.green('| .` / _` |  _| \\ V / -_)'));
    console.log(chalk_1.default.yellow('|___|_\\___\\__|\\__|_| \\___/\\__,_\\___| ') +
        chalk_1.default.green('|_|\\_\\__,_|\\__|_|\\_/\\___|'));
}
async function showInfo() {
    const currentCauldronRepo = ern_core_1.config.get('cauldronRepoInUse') || '-NONE-';
    const bundleStoreId = ern_core_1.config.get('bundlestore-id');
    const l = bundleStoreId ? chalk_1.default.cyan(` [BundleStore: ${bundleStoreId}]`) : '';
    console.log(chalk_1.default.cyan(`[v${ern_core_1.Platform.currentVersion}]`) +
        chalk_1.default.cyan(` [Cauldron: ${currentCauldronRepo}]`) +
        l);
    if (ern_core_1.ERN_RC_GLOBAL_FILE_PATH !== ern_core_1.ernRcFilePath) {
        // Log .ernrc configuration file path only if it differs from default
        console.log(chalk_1.default.cyan(`${chalk_1.default.bold('Config:')} ${ern_core_1.ernRcFilePath}`));
    }
    const overrideManifest = await ern_core_1.Manifest.getOverrideManifestConfig();
    if (overrideManifest) {
        // Log override manifest only if set
        console.log(chalk_1.default.cyan(`${chalk_1.default.bold('Manifest [override]:')} ${overrideManifest.url} ${chalk_1.default.italic(`(from ${overrideManifest.source}`)})`));
    }
    const manifestErnRcConf = ern_core_1.config.get('manifest', undefined);
    if (manifestErnRcConf && manifestErnRcConf.master) {
        //  Log master manifest only if set
        console.log(chalk_1.default.cyan(`${chalk_1.default.bold('Manifest [master]:')} ${manifestErnRcConf.master.url} ${chalk_1.default.italic('(from .ernrc)')}`));
    }
    console.log('');
}
function showVersion() {
    let packageInfo;
    try {
        const pkg = path_1.default.join((0, child_process_1.execSync)('yarn global dir').toString().trim(), 'node_modules', 'electrode-native', 'package.json');
        packageInfo = fs_extra_1.default.existsSync(pkg) ? require(pkg) : '';
    }
    catch (err) {
        ern_core_1.log.trace(err);
    }
    if (!packageInfo) {
        try {
            packageInfo = JSON.parse((0, child_process_1.execSync)('npm ls -g electrode-native --json').toString()).dependencies['electrode-native'];
        }
        catch (err) {
            ern_core_1.log.trace(err);
        }
    }
    const globalVersion = packageInfo ? packageInfo.version : '';
    ern_core_1.log.info(`electrode-native: ${globalVersion ? globalVersion : '-UNKNOWN-'}`);
    const localVersion = ern_core_1.config.get('platformVersion');
    ern_core_1.log.info(`ern platform: ${localVersion ? localVersion : '-UNKNOWN-'}`);
}
function getNodeEngineVersionRequirement() {
    const pjsonPath = path_1.default.resolve(__dirname, '..', 'package.json');
    const pjson = fs_extra_1.default.readJSONSync(pjsonPath);
    return pjson.engines.node;
}
function isNodeVersionCompatible(version) {
    const requiredVersion = getNodeEngineVersionRequirement();
    return semver_1.default.satisfies(version, requiredVersion);
}
ern_core_1.Manifest.getOverrideManifestConfig =
    async () => {
        // Try to find override manifest config in .ernrc config first
        let manifestConfig = ern_core_1.config.get('manifest', undefined);
        if (manifestConfig &&
            manifestConfig.override &&
            manifestConfig.override.url) {
            return {
                source: '.ernrc',
                type: manifestConfig.override.type || 'partial',
                url: manifestConfig.override.url,
            };
        }
        // If not found in .ernrc, look in cauldron config
        try {
            const cauldronInstance = await (0, ern_cauldron_api_1.getActiveCauldron)({
                silent: true,
                throwIfNoActiveCauldron: false,
            });
            manifestConfig = await (cauldronInstance === null || cauldronInstance === void 0 ? void 0 : cauldronInstance.getManifestConfig());
            if (manifestConfig) {
                return {
                    source: 'cauldron',
                    type: manifestConfig.override.type || 'partial',
                    url: manifestConfig.override.url,
                };
            }
        }
        catch (e) {
            ern_core_1.log.warn('Cannot reach Cauldron');
        }
    };
const kaxRendererConfig = {
    colorScheme: {
        error: 'red',
        info: 'cyan',
        task: 'white',
        warning: 'yellow',
    },
    shouldLogTime: true,
    symbolScheme: {
        error: 'error',
        info: 'info',
        taskFailure: 'error',
        taskRunning: 'dots',
        taskSuccess: 'success',
        warning: 'warning',
    },
    symbolizeMultiLine: true,
};
class KaxNullRenderer {
    renderWarning(msg) {
        // noop
    }
    renderInfo(msg) {
        // noop
    }
    renderError(msg) {
        // noop
    }
    renderRaw(msg) {
        // noop
    }
    renderTask(msg, task) {
        // noop
    }
}
const logLevelStringToEnum = (level) => {
    switch (level) {
        case 'trace':
            return ern_core_1.LogLevel.Trace;
        case 'debug':
            return ern_core_1.LogLevel.Debug;
        case 'info':
            return ern_core_1.LogLevel.Info;
        case 'warn':
            return ern_core_1.LogLevel.Warn;
        case 'error':
            return ern_core_1.LogLevel.Error;
        case 'off':
            return ern_core_1.LogLevel.Off;
        default:
            throw new Error(`Invalid log level ${level}`);
    }
};
// ==============================================================================
// Entry point
// =============================================================================
(async function run() {
    const hasJsonOpt = process.argv.slice(1).includes('--json');
    const logLevel = hasJsonOpt
        ? ern_core_1.LogLevel.Off
        : process.env.ERN_LOG_LEVEL
            ? logLevelStringToEnum(process.env.ERN_LOG_LEVEL)
            : logLevelStringToEnum(ern_core_1.config.get('logLevel', 'info'));
    ern_core_1.log.setLogLevel(logLevel);
    ern_core_1.shell.config.fatal = true;
    ern_core_1.shell.config.verbose = logLevel === ern_core_1.LogLevel.Trace;
    ern_core_1.shell.config.silent = !(logLevel === ern_core_1.LogLevel.Trace || logLevel === ern_core_1.LogLevel.Debug);
    ern_core_1.kax.renderer =
        logLevel === ern_core_1.LogLevel.Trace || logLevel === ern_core_1.LogLevel.Debug
            ? new kax_1.KaxSimpleRenderer(kaxRendererConfig)
            : logLevel === ern_core_1.LogLevel.Off
                ? new KaxNullRenderer()
                : new kax_1.KaxAdvancedRenderer(kaxRendererConfig);
    const nodeVersion = process.version;
    if (!isNodeVersionCompatible(nodeVersion)) {
        const requiredVersion = getNodeEngineVersionRequirement();
        ern_core_1.log.error(`This version of ern requires Node.js ${requiredVersion}
Node.js version currently in use is ${nodeVersion}
Please switch to a version of Node satisfying the version requirement`);
        process.exit(1);
    }
    if (!hasJsonOpt) {
        if (ern_core_1.config.get('showBanner', true)) {
            showBanner();
        }
        await showInfo();
    }
    if (process.argv[2] === '--version') {
        return showVersion();
    }
    return yargs_1.default
        .commandDir('commands', {
        extensions: process.env.ERN_ENV === 'development' ? ['js', 'ts'] : ['js'],
    })
        .demandCommand()
        .help()
        .strict()
        .version(false)
        .wrap(yargs_1.default.terminalWidth()).argv;
})();
//# sourceMappingURL=index.js.map